#include <stdio.h> 
#include "nexus.h"
#include "gram_client.h"

static void callback_func(void * user_callback_arg,
                          char * job_contact,
                          int state,
                          int errorcode);

typedef struct
{
    nexus_mutex_t mutex;
    nexus_cond_t cond;
    nexus_bool_t done;
} monitor_t;

static monitor_t                Monitor;

int main(int argc, char ** argv)
{
    int callback_fd;
    int job_state_mask;
    int test_int = 55;
    int rc;
    char * callback_contact;
    char * job_contact;
    char * rm_contact;
    char * specification;
    float confidence;
    gram_bool_t done;
    gram_time_t estimate;
    gram_time_t interval_size;

    /* 
    printf("argc   = %d\n", argc);
    printf("argv 0 = %s\n", argv[0]);
    printf("rm contact = %s\n", argv[1]);
    printf("specification = %s\n", argv[2]);
    printf("job state mask = %s\n", argv[3]);
    */

    if (argc != 4 && argc != 5)
    {
        /* invalid parameters passed */
        printf("Usage: %s <rm_contact> <specification> <job_state_mask> <-debug>\n",
                argv[0]);
        exit(1);
    }

    gram_init(&argc, &argv);

    rm_contact = (char *)malloc(strlen(argv[1])+1);
    strcpy(rm_contact, argv[1]);
    specification = (char *)malloc(strlen(argv[2])+1);
    strcpy(specification, argv[2]);
    job_state_mask = atoi(argv[3]);

    nexus_mutex_init(&Monitor.mutex, (nexus_mutexattr_t *) NULL);
    nexus_cond_init(&Monitor.cond, (nexus_condattr_t *) NULL);

    /* entering the monitor and clearing the flag */
    nexus_mutex_lock(&Monitor.mutex);
    Monitor.done = NEXUS_FALSE;
    nexus_mutex_unlock(&Monitor.mutex);

    gram_callback_allow(callback_func,
                        (void *) test_int,
                       &callback_contact);

    printf("callback contact = %s\n", callback_contact);

    rc = gram_job_request(rm_contact,
                         specification,
	                 job_state_mask,
		         callback_contact,
                         &job_contact);

    printf("return value = %i\n", rc);

    if (rc != 0)
    {
        printf("error submitting job.\n");
        exit(1);
    }
    else
    {
        printf("job contact  = %s\n", job_contact);
    }

#ifdef USE_START_TIME
    confidence = .7;
    gram_job_start_time(job_contact, confidence, &estimate, &interval_size);
    printf("estimate = %d\n", estimate.dumb_time);
    printf("interval = %d\n", interval_size.dumb_time);
#endif

#ifdef CANCEL
    if (gram_job_cancel(job_contact) != 0)
    {
       printf("failed to send cancel to job manager\n");
       printf("test failed.\n");
       return(1);
    }
#endif
    nexus_mutex_lock(&Monitor.mutex);
    while (!Monitor.done)
    {
        nexus_cond_wait(&Monitor.cond, &Monitor.mutex);
    } /* endwhile */
    nexus_mutex_unlock(&Monitor.mutex);

    nexus_mutex_destroy(&Monitor.mutex);
    nexus_cond_destroy(&Monitor.cond);

    gram_job_contact_free(job_contact);

    gram_shutdown();

    printf("test completed.\n");

    return(0);
}

/*
void callback_check_func(void * arg, int fd)
{
    gram_callback_check(fd);
}
*/

void callback_func(void * user_callback_arg,
                   char * job_contact,
                   int state,
                   int errorcode)
{
    int test_int = (int) user_callback_arg;

    printf("test int = %d.\n", test_int);
 
    printf("callback_func(): job_contact=%s, state=%i, errorcode=%i\n", 
           job_contact, state, errorcode);

    switch(state)
    {
    case GRAM_JOB_STATE_PENDING:
	printf("Got GRAM_JOB_STATE_PENDING\n");
	break;
    case GRAM_JOB_STATE_ACTIVE:
	printf("Got GRAM_JOB_STATE_ACTIVE\n");
	break;
    case GRAM_JOB_STATE_FAILED:
	printf("Got GRAM_JOB_STATE_FAILED\n");
        nexus_mutex_lock(&Monitor.mutex);
        Monitor.done = NEXUS_TRUE;
        nexus_cond_signal(&Monitor.cond);
        nexus_mutex_unlock(&Monitor.mutex);
	break;
    case GRAM_JOB_STATE_DONE:
	printf("Got GRAM_JOB_STATE_DONE\n");
        nexus_mutex_lock(&Monitor.mutex);
        Monitor.done = NEXUS_TRUE;
        nexus_cond_signal(&Monitor.cond);
        nexus_mutex_unlock(&Monitor.mutex);
	break;
    }
}
